package jeux.ressources;

import java.net.URL;
import java.util.Hashtable;

/**
 * @author jub
 *
 * @param <R> Le type gr par le gestionnaire de ressouce <br>
 * Un gestionnaire de ressource gnrique et abstrait qui nous permettra de grer nos ressources. <br>
 * Les ressources une fois charges sont stoques dans un HashTable et ceux pour augementer la rapidit en limitant les accs disque. 
 */
public abstract class RessourceCache<R> {
	/**
	 * Une collection pour stocker les ressources et y accder via leur nom.
	 */
	private Hashtable<String,R> ressources;

	/**
	 * Construit un gestionnqire de ressource vide<
	 */
	public RessourceCache() {
		ressources = new Hashtable<String,R>(); 
	}
	
	/**
	 * @param url l'URL de la ressource 
	 * @return la ressource
	 */
	public abstract R loadRessource(URL url);
	
	/**
	 * @param name une chane de caractre qui reprsente la ressource. 
	 * @return la ressource
	 */
	public R loadRessource(String name)
	{
		URL url = null;
		url = this.getClass().getClassLoader().getResource(name);
		return this.loadRessource(url);
	}
	
	/**
	 * @param name une chane de caractre qui reprsente la ressource. 
	 * @return la ressource
	 * Renvoie la ressource depuis la mmoire si elle est dj charge et sinon la charge depuis le disque
	 */
	public R getRessource(String name)
	{
		R r = ressources.get(name);
		if (r == null)
		{
			r = loadRessource(name);
			ressources.put(name, r);
		}
		return r;
		
	}
	
	/**
	 * Libre la mmoire utilise.
	 */
	public void flush(){
		ressources = new Hashtable<String,R>(); 
	}
}
